function [] = custom_one_sample_T_test_programmatic(input_cell,base,dof_map,subj_thresh,output_dir,test_label)
%custom_one_sample_T_test_programmatic performs a one sample T-test on the
%input data. This T-test is based off of DPABI's one sample T-test method,
%which uses QR decomposition to prepare the data for the T-test. This
%function performs the T-test using pairwise deletion, in which the T-test
%is performed on every voxel, adjusting the degrees of freedom for voxels
%in which not every subject has data. This is in contrast to the usual
%method of listwise deletion, in which voxels where not every subject has
%data are removed from analysis.
%
%Inputs:
%   input_cell  =   an Nx1 cell array with the paths to the images that the
%                   T-test will be performed on 
%   base        =   the mean/bias of the data. Is subtracted  out before
%                   QR decomposition and the T-test
%   dof_map     =   a single image that defines the number of subjects have
%                   valid data at each voxel. Is the output
%                   overlap_map_programmatic.m
%   subj_thresh =   a minimum threshold of the number of subjects for a
%                   voxel to be analyzed. If there are too few subjects
%                   with data at a given voxel, the dof will be very low,
%                   and the output of the T-test will be inflated
%                   accordingly
%   output_dir  =   the name of the output directory
%   test_label  =   a prefix that will be written to the name of each
%                   output file
tot_sub = size(input_cell,1);

for vv = 1:tot_sub
    if vv == 1
        dep_vol = spm_read_vols(spm_vol(input_cell{vv}));
        V = spm_vol(input_cell{vv});
    else
        dep_vol = cat(4,dep_vol,spm_read_vols(spm_vol(input_cell{vv})));
    end
end

nVol = spm_read_vols(spm_vol(dof_map));
nVol = round(nVol);

[x,y,z,subj] = size(dep_vol);
dep_vol = dep_vol-base;
X = ones(subj,1);
bVol = zeros(size(dep_vol(:,:,:,1)));
sseVol = zeros(size(bVol));
ssrVol = zeros(size(bVol));
tVol = zeros(size(bVol));

for xx = 1:x
    for yy = 1:y
        for zz = 1:z
            nSubjs = nVol(xx,yy,zz);
            if nSubjs < subj_thresh
                bVol(xx,yy,zz) = 0;
            else
                response = squeeze(dep_vol(xx,yy,zz,:));
                [xmod,ymod] = rm_empty_subj(X,response,nSubjs);
                [Q,R,perm] = qr(xmod,0);
                p = sum(abs(diag(R))) > max(size(xmod,1),size(xmod,2))*eps(R(1));
                if p < size(xmod,2)
                    R = R(1:p,1:p);
                    Q = Q(:,1:p);
                    perm = perm(1:p);
                end
                b = zeros(size(xmod,2),1);
                b(perm) = R\(Q'*ymod);
                bVol(xx,yy,zz) = b;
                yhat = xmod*b;
                resid = ymod-yhat;
                sseVol(xx,yy,zz) = sum(resid.^2);
                ssrVol(xx,yy,zz) = sum((yhat-mean(y)).^2);
                [~,R] = qr(xmod,0);
                ri = R\eye(size(xmod,2));
                tVol(xx,yy,zz) = b./sqrt(diag(ri*ri'*(sseVol(xx,yy,zz)/(nSubjs-size(xmod,2)))));
            end
        end
    end
end

if exist(output_dir,'dir') ~= 7
    mkdir(output_dir);
end

% tV = V;
% tV.fname = [output_dir 'T_' test_label '.nii'];
% tV.descrip = 'T-score Volume with variable DOF';
% spm_write_vol(tV,tVol);

pVol = tcdf(tVol,nVol-1);
pV = V;
pV.fname = [output_dir 'P_' test_label '.nii'];
pV.descrip = 'P-value Volume with variable DOF';
spm_write_vol(pV,pVol);

DOFmax = (tot_sub-1)*ones(size(pVol));
t_equiv_Vol = tinv(pVol,DOFmax);
t_equiv_V = V;
t_equiv_V.fname = [output_dir 'T_equivalent_' test_label '.nii'];
t_equiv_V.descrip = ['T-equivalent score relative to ' num2str(tot_sub-1) ' degrees of freedom.'];
spm_write_vol(t_equiv_V,t_equiv_Vol);

end